==> fred_regal-1.2.0-py3-none-any.whl: regal/contact.py <==
"""Contact client for registry service."""
from datetime import date, datetime
from enum import IntEnum, unique
from typing import Any, Dict, List, Mapping, Optional, Union

import fred_api.registry.contact.service_contact_grpc_pb2_grpc
import grpc
from backports.strenum import StrEnum
from fred_api.registry.contact.contact_history_types_pb2 import ContactHistoryReply, ContactHistoryRequest
from fred_api.registry.contact.contact_info_types_pb2 import (
    ContactAdditionalIdentifier as AdditionalIdentifierMessage,
    ContactIdReply,
    ContactIdRequest,
    ContactInfoReply,
    ContactInfoRequest,
    WarningLetter as WarningLetterMessage,
)
from fred_api.registry.contact.contact_state_history_types_pb2 import (
    ContactStateHistoryReply,
    ContactStateHistoryRequest,
)
from fred_api.registry.contact.contact_state_types_pb2 import ContactStateReply, ContactStateRequest
from fred_api.registry.contact.update_contact_state_types_pb2 import UpdateContactStateReply, UpdateContactStateRequest
from fred_types import BaseModel, ContactHistoryId, ContactId, LogEntryId
from pydantic import root_validator

from .common import Address, ObjectEvents, RegistryDecoder, _set_sponsoring_registrar
from .exceptions import ContactDoesNotExist, InvalidHistoryInterval, NotCurrentVersion
from .object import RegistryObjectClient


@unique
class WarningLetterPreference(IntEnum):
    """Enum with warning letter results."""

    TO_SEND = 0
    NOT_TO_SEND = 1
    NOT_SPECIFIED = 2


@unique
class ContactIdentifierType(StrEnum):
    """Contact identifier type enum."""

    NATIONAL_IDENTITY_NUMBER = "national_identity_number"
    NATIONAL_IDENTITY_CARD = "national_identity_card"
    PASSPORT_NUMBER = "passport_number"
    COMPANY_REGISTRATION_NUMBER = "company_registration_number"
    SOCIAL_SECURITY_NUMBER = "social_security_number"
    BIRTHDATE = "birthdate"


class AdditionalIdentifier(BaseModel):
    """Additional contact identifier."""

    type: ContactIdentifierType
    value: str

    @property
    def birthdate(self) -> Optional[date]:
        """Return birthdate as a date if possible.

        Returns: Birthdate as a date object or None if the identifier is not a birthdate.

        Raises:
            ValueError: If the identifier has birthdate type, but invalid value.
                        Legacy data in the registry may not have a valid format.
                        Such cases must be handled separately, according to the application.
        """
        if self.type != ContactIdentifierType.BIRTHDATE or not self.value:
            return None
        try:
            return date.fromisoformat(self.value)
        except ValueError as error:
            raise ValueError("{!r} is not a valid date.".format(self.value)) from error


class Contact(BaseModel):
    """Contact info."""

    contact_id: ContactId
    contact_handle: str
    contact_history_id: Optional[ContactHistoryId] = None
    sponsoring_registrar: Optional[str] = None
    name: str = ""
    organization: str = ""
    place: Optional[Address] = None
    telephone: Optional[str] = None
    fax: Optional[str] = None
    emails: List[str] = []
    notify_emails: List[str] = []
    vat_identification_number: Optional[str] = None
    additional_identifier: Optional[AdditionalIdentifier] = None
    publish: Dict[str, bool] = {}
    mailing_address: Optional[Address] = None
    billing_address: Optional[Address] = None
    shipping_address1: Optional[Address] = None
    shipping_address2: Optional[Address] = None
    shipping_address3: Optional[Address] = None
    warning_letter: WarningLetterPreference = WarningLetterPreference.NOT_SPECIFIED
    auth_info: Optional[str] = None
    events: Optional[ObjectEvents] = None

    _set_sponsoring_registrar = root_validator(pre=True, allow_reuse=True)(_set_sponsoring_registrar)


class ContactDecoder(RegistryDecoder):
    """Contact service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(AdditionalIdentifierMessage, self._decode_additional_identifier)
        self.set_decoder(WarningLetterMessage, self._decode_warning_letter)
        self.set_exception_decoder(ContactIdReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactInfoReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactHistoryReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(ContactStateReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactStateHistoryReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactStateHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(UpdateContactStateReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(UpdateContactStateReply.Exception.NotCurrentVersion, NotCurrentVersion)

    def _decode_additional_identifier(self, value: AdditionalIdentifierMessage) -> Optional[AdditionalIdentifier]:
        """Decode additional identifier."""
        field = value.WhichOneof("Variant")
        if field is not None:
            return AdditionalIdentifier(type=ContactIdentifierType(field), value=self.decode(getattr(value, field)))
        else:
            return None

    def _decode_warning_letter(self, value: WarningLetterMessage) -> WarningLetterPreference:
        """Decode WarningLetter."""
        return WarningLetterPreference(value.preference)


class ContactClient(RegistryObjectClient[ContactId, ContactHistoryId]):
    """Contact service gRPC client."""

    service = "Contact"
    object_id = "contact_id"
    object_history_id = "contact_history_id"
    object_handle = "contact_handle"

    def __init__(self, netloc: str, credentials: Optional[grpc.ChannelCredentials] = None) -> None:
        """Initialize contact client instance.

        Args:
            netloc: Netloc of contact service server.
            credentials: Optional credentials.
        """
        super().__init__(
            netloc,
            decoder=ContactDecoder(),
            grpc_modules=[fred_api.registry.contact.service_contact_grpc_pb2_grpc],
            credentials=credentials,
        )

    async def get_contact_id(self, handle: str) -> ContactId:
        """Get contact id.

        Args:
            handle: Contact handle.

        Raises:
            ContactDoesNotExist: If contact does not exist.
        """
        return ContactId(await self.get_object_id(request=ContactIdRequest(), method="get_contact_id", handle=handle))

    async def get_contact_info(self, contact_id: ContactId, history_id: Optional[ContactHistoryId] = None) -> Contact:
        """Get contact info.

        Args:
            contact_id: Contact ID.
            history_id: Contact history ID.

        Raises:
            ContactDoesNotExist: If contact does not exist.
        """
        result = await self.get_object_info(
            request=ContactInfoRequest(),
            method="get_contact_info",
            object_id=contact_id,
            history_id=history_id,
        )
        return Contact(**result)

    async def get_contact_state(self, contact_id: ContactId) -> Dict[str, bool]:
        """Get contact state consisting of state flags and their boolean values.

        Raises:
            ContactDoesNotExist: If contact does not exist.

        Args:
            contact_id: Contact ID.
        """
        return await self.get_object_state(
            request=ContactStateRequest(),
            method="get_contact_state",
            object_id=contact_id,
        )

    async def get_contact_history(
        self,
        contact_id: ContactId,
        start: Optional[Union[ContactHistoryId, datetime]] = None,
        end: Optional[Union[ContactHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get contact history timeline.

        Args:
            contact_id: Contact ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            ContactDoesNotExist: If contact does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_history(
            request=ContactHistoryRequest(),
            method="get_contact_history",
            object_id=contact_id,
            start=start,
            end=end,
        )

    async def get_contact_state_history(
        self,
        contact_id: ContactId,
        start: Optional[Union[ContactHistoryId, datetime]] = None,
        end: Optional[Union[ContactHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get contact state history timeline.

        Args:
            contact_id: Contact ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            ContactDoesNotExist: If contact does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_state_history(
            request=ContactStateHistoryRequest(),
            method="get_contact_state_history",
            object_id=contact_id,
            start=start,
            end=end,
        )

    async def update_contact_state(
        self,
        contact_id: ContactId,
        history_id: ContactHistoryId,
        set_flags: Mapping[str, bool],
        log_entry_id: Optional[LogEntryId] = None,
    ) -> None:
        """Update contact's state.

        Args:
            contact_id: Contact identification.
            history_id: Contact history identification.
            set_flags: Set of flags (name and new value) to update.
            log_entry_id: Identification of logger entry for the action.

        Returns:
            none: Operation successfully finished.

        Raises:
            ContactDoesNotExist: Contact not found.
            NotCurrentVersion: Contact history identification not valid.
        """
        request = UpdateContactStateRequest()
        request.contact_id.uuid.value = contact_id
        request.contact_history_id.uuid.value = history_id
        for key, value in set_flags.items():
            request.set_flags[key] = value
        if log_entry_id is not None:
            request.log_entry_id.value = log_entry_id
        await self.call(self.service, "update_contact_state", request)

==> fred_regal-1.3.0-py3-none-any.whl: regal/contact.py <==
"""Contact client for registry service."""
from datetime import date, datetime
from enum import IntEnum, unique
from typing import Any, Dict, List, Mapping, Optional, Union

import fred_api.registry.contact.service_contact_grpc_pb2_grpc
import grpc
from backports.strenum import StrEnum
from fred_api.registry.contact.contact_history_types_pb2 import ContactHistoryReply, ContactHistoryRequest
from fred_api.registry.contact.contact_info_types_pb2 import (
    ContactAdditionalIdentifier as AdditionalIdentifierMessage,
    ContactIdReply,
    ContactIdRequest,
    ContactInfoReply,
    ContactInfoRequest,
    WarningLetter as WarningLetterMessage,
)
from fred_api.registry.contact.contact_state_history_types_pb2 import (
    ContactStateHistoryReply,
    ContactStateHistoryRequest,
)
from fred_api.registry.contact.contact_state_types_pb2 import ContactStateReply, ContactStateRequest
from fred_api.registry.contact.update_contact_state_types_pb2 import UpdateContactStateReply, UpdateContactStateRequest
from fred_types import BaseModel, ContactHistoryId, ContactId, LogEntryId

try:
    from pydantic.v1 import root_validator
except ImportError:  # pragma: no cover
    from pydantic import root_validator  # type: ignore[no-redef]

from .common import Address, ObjectEvents, RegistryDecoder, _set_sponsoring_registrar
from .exceptions import ContactDoesNotExist, InvalidHistoryInterval, NotCurrentVersion
from .object import RegistryObjectClient


@unique
class WarningLetterPreference(IntEnum):
    """Enum with warning letter results."""

    TO_SEND = 0
    NOT_TO_SEND = 1
    NOT_SPECIFIED = 2


@unique
class ContactIdentifierType(StrEnum):
    """Contact identifier type enum."""

    NATIONAL_IDENTITY_NUMBER = "national_identity_number"
    NATIONAL_IDENTITY_CARD = "national_identity_card"
    PASSPORT_NUMBER = "passport_number"
    COMPANY_REGISTRATION_NUMBER = "company_registration_number"
    SOCIAL_SECURITY_NUMBER = "social_security_number"
    BIRTHDATE = "birthdate"


class AdditionalIdentifier(BaseModel):
    """Additional contact identifier."""

    type: ContactIdentifierType
    value: str

    @property
    def birthdate(self) -> Optional[date]:
        """Return birthdate as a date if possible.

        Returns: Birthdate as a date object or None if the identifier is not a birthdate.

        Raises:
            ValueError: If the identifier has birthdate type, but invalid value.
                        Legacy data in the registry may not have a valid format.
                        Such cases must be handled separately, according to the application.
        """
        if self.type != ContactIdentifierType.BIRTHDATE or not self.value:
            return None
        try:
            return date.fromisoformat(self.value)
        except ValueError as error:
            raise ValueError("{!r} is not a valid date.".format(self.value)) from error


class Contact(BaseModel):
    """Contact info."""

    contact_id: ContactId
    contact_handle: str
    contact_history_id: Optional[ContactHistoryId] = None
    sponsoring_registrar: Optional[str] = None
    name: str = ""
    organization: str = ""
    place: Optional[Address] = None
    telephone: Optional[str] = None
    fax: Optional[str] = None
    emails: List[str] = []
    notify_emails: List[str] = []
    vat_identification_number: Optional[str] = None
    additional_identifier: Optional[AdditionalIdentifier] = None
    publish: Dict[str, bool] = {}
    mailing_address: Optional[Address] = None
    billing_address: Optional[Address] = None
    shipping_address1: Optional[Address] = None
    shipping_address2: Optional[Address] = None
    shipping_address3: Optional[Address] = None
    warning_letter: WarningLetterPreference = WarningLetterPreference.NOT_SPECIFIED
    auth_info: Optional[str] = None
    events: Optional[ObjectEvents] = None

    _set_sponsoring_registrar = root_validator(pre=True, allow_reuse=True)(_set_sponsoring_registrar)


class ContactDecoder(RegistryDecoder):
    """Contact service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(AdditionalIdentifierMessage, self._decode_additional_identifier)
        self.set_decoder(WarningLetterMessage, self._decode_warning_letter)
        self.set_exception_decoder(ContactIdReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactInfoReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactHistoryReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(ContactStateReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactStateHistoryReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(ContactStateHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(UpdateContactStateReply.Exception.ContactDoesNotExist, ContactDoesNotExist)
        self.set_exception_decoder(UpdateContactStateReply.Exception.NotCurrentVersion, NotCurrentVersion)

    def _decode_additional_identifier(self, value: AdditionalIdentifierMessage) -> Optional[AdditionalIdentifier]:
        """Decode additional identifier."""
        field = value.WhichOneof("Variant")
        if field is not None:
            return AdditionalIdentifier(type=ContactIdentifierType(field), value=self.decode(getattr(value, field)))
        else:
            return None

    def _decode_warning_letter(self, value: WarningLetterMessage) -> WarningLetterPreference:
        """Decode WarningLetter."""
        return WarningLetterPreference(value.preference)


class ContactClient(RegistryObjectClient[ContactId, ContactHistoryId]):
    """Contact service gRPC client."""

    service = "Contact"
    object_id = "contact_id"
    object_history_id = "contact_history_id"
    object_handle = "contact_handle"

    def __init__(self, netloc: str, credentials: Optional[grpc.ChannelCredentials] = None) -> None:
        """Initialize contact client instance.

        Args:
            netloc: Netloc of contact service server.
            credentials: Optional credentials.
        """
        super().__init__(
            netloc,
            decoder=ContactDecoder(),
            grpc_modules=[fred_api.registry.contact.service_contact_grpc_pb2_grpc],
            credentials=credentials,
        )

    async def get_contact_id(self, handle: str) -> ContactId:
        """Get contact id.

        Args:
            handle: Contact handle.

        Raises:
            ContactDoesNotExist: If contact does not exist.
        """
        return ContactId(await self.get_object_id(request=ContactIdRequest(), method="get_contact_id", handle=handle))

    async def get_contact_info(self, contact_id: ContactId, history_id: Optional[ContactHistoryId] = None) -> Contact:
        """Get contact info.

        Args:
            contact_id: Contact ID.
            history_id: Contact history ID.

        Raises:
            ContactDoesNotExist: If contact does not exist.
        """
        result = await self.get_object_info(
            request=ContactInfoRequest(),
            method="get_contact_info",
            object_id=contact_id,
            history_id=history_id,
        )
        return Contact(**result)

    async def get_contact_state(self, contact_id: ContactId) -> Dict[str, bool]:
        """Get contact state consisting of state flags and their boolean values.

        Raises:
            ContactDoesNotExist: If contact does not exist.

        Args:
            contact_id: Contact ID.
        """
        return await self.get_object_state(
            request=ContactStateRequest(),
            method="get_contact_state",
            object_id=contact_id,
        )

    async def get_contact_history(
        self,
        contact_id: ContactId,
        start: Optional[Union[ContactHistoryId, datetime]] = None,
        end: Optional[Union[ContactHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get contact history timeline.

        Args:
            contact_id: Contact ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            ContactDoesNotExist: If contact does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_history(
            request=ContactHistoryRequest(),
            method="get_contact_history",
            object_id=contact_id,
            start=start,
            end=end,
        )

    async def get_contact_state_history(
        self,
        contact_id: ContactId,
        start: Optional[Union[ContactHistoryId, datetime]] = None,
        end: Optional[Union[ContactHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get contact state history timeline.

        Args:
            contact_id: Contact ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            ContactDoesNotExist: If contact does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_state_history(
            request=ContactStateHistoryRequest(),
            method="get_contact_state_history",
            object_id=contact_id,
            start=start,
            end=end,
        )

    async def update_contact_state(
        self,
        contact_id: ContactId,
        history_id: ContactHistoryId,
        set_flags: Mapping[str, bool],
        log_entry_id: Optional[LogEntryId] = None,
    ) -> None:
        """Update contact's state.

        Args:
            contact_id: Contact identification.
            history_id: Contact history identification.
            set_flags: Set of flags (name and new value) to update.
            log_entry_id: Identification of logger entry for the action.

        Returns:
            none: Operation successfully finished.

        Raises:
            ContactDoesNotExist: Contact not found.
            NotCurrentVersion: Contact history identification not valid.
        """
        request = UpdateContactStateRequest()
        request.contact_id.uuid.value = contact_id
        request.contact_history_id.uuid.value = history_id
        for key, value in set_flags.items():
            request.set_flags[key] = value
        if log_entry_id is not None:
            request.log_entry_id.value = log_entry_id
        await self.call(self.service, "update_contact_state", request)
